/*
 * Copyright (c) 1997  Motoyuki Kasahara
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef EB_TEXT_H
#define EB_TEXT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>

#ifdef EB_BUILD_LIBRARY
#include "types.h"
#else
#include <eb/types.h>
#endif


/*
 * Hook codes.
 */
#define EB_HOOK_BEGIN_NARROW		0
#define EB_HOOK_NARROW			1
#define EB_HOOK_END_NARROW		2
#define EB_HOOK_BEGIN_SUBSCRIPT		3
#define EB_HOOK_SUBSCRIPT		4
#define EB_HOOK_END_SUBSCRIPT		5
#define EB_HOOK_SET_INDENT		6
#define EB_HOOK_NEWLINE			7
#define EB_HOOK_BEGIN_SUPERSCRIPT	8
#define EB_HOOK_SUPERSCRIPT		9
#define EB_HOOK_END_SUPERSCRIPT		10
#define EB_HOOK_BEGIN_TABLE		11
#define EB_HOOK_END_TABLE		12
#define EB_HOOK_BEGIN_NO_NEWLINE	13
#define EB_HOOK_NO_NEWLINE		14
#define EB_HOOK_END_NO_NEWLINE		15
#define EB_HOOK_BEGIN_EMPHASIS		16
#define EB_HOOK_EMPHASIS		17
#define EB_HOOK_END_EMPHASIS		18
#define EB_HOOK_BEGIN_PICTURE		19
#define EB_HOOK_PICTURE			20
#define EB_HOOK_END_PICTURE		21
#define EB_HOOK_BEGIN_MENU		22
#define EB_HOOK_MENU			23
#define EB_HOOK_END_MENU		24
#define EB_HOOK_BEGIN_SOUND		25
#define EB_HOOK_SOUND			26
#define EB_HOOK_END_SOUND		27
#define EB_HOOK_BEGIN_REFERENCE		28
#define EB_HOOK_REFERENCE		29
#define EB_HOOK_END_REFERENCE		30
#define EB_HOOK_BEGIN_KEYWORD		31
#define EB_HOOK_KEYWORD			32
#define EB_HOOK_END_KEYWORD		33
#define EB_HOOK_ISO8859_1		34
#define EB_HOOK_JISX0208		35
#define EB_HOOK_NARROW_FONT		36
#define EB_HOOK_WIDE_FONT		37
#define EB_HOOK_OTHERS			38
#define EB_HOOK_UNKNOWN_SEQUENCE	39
#define EB_HOOK_INITIALIZE		40
#define EB_HOOK_STOPCODE		41
#define EB_HOOK_NULL			42


/*
 * The number of hooks.
 */
#define EB_NUM_HOOKS		43

/*
 * The maximum length of a string in the text work buffer.
 */
#define EB_MAXLEN_TEXT_WORK	255


/*
 * Type for hook code.
 */
typedef int EB_Hook_Code;


/*
 * EB_Hook -- A text hook.
 */
typedef struct {
    EB_Hook_Code code;
#ifdef __STDC__
    int (*function)(EB_Book *, EB_Appendix *, char *, EB_Hook_Code, int,
	const int *);
#else /* not __STDC__ */
    int (*function)();
#endif /* __STDC__ */
} EB_Hook;


/*
 * EB_Hookset -- A set of text hooks.
 */
typedef struct {
    EB_Hook hooks[EB_NUM_HOOKS];
} EB_Hookset;


/*
 * Function declarations.
 */
#ifdef __STDC__

/* hook.c */
int eb_hook_euc_to_ascii(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
int eb_hook_stopcode(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
int eb_hook_stopcode_dummy(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
int eb_hook_stopcode_mixed(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
/* text.c */
void eb_initialize_hookset(EB_Hookset *);
int eb_set_hook(EB_Hookset *, const EB_Hook *);
int eb_set_hooks(EB_Hookset *, const EB_Hook *);
int eb_heading(EB_Book *, EB_Appendix *, const EB_Hookset *,
    const EB_Position *, char *, size_t);
int eb_next_heading(EB_Book *, EB_Appendix *, const EB_Hookset *,
    char *, size_t);
int eb_text(EB_Book *, EB_Appendix *, const EB_Hookset *,
    const EB_Position *, char *, size_t);
int eb_next_text(EB_Book *, EB_Appendix *, const EB_Hookset *,
    char *, size_t);

#else /* not __STDC__ */

/* hook.c */
int eb_hook_euc_to_ascii();
int eb_hook_stopcode();
int eb_hook_stopcode_dummy();
int eb_hook_stopcode_mixed();
/* text.c */
void eb_initialize_hookset();
int eb_set_hook();
int eb_set_hooks();
int eb_heading();
int eb_next_heading();
int eb_text();
int eb_next_text();

#endif /* not __STDC__ */


#ifdef __cplusplus
}
#endif

#endif /* not EB_TEXT_H */
