/*
 * Copyright (c) 1997  Motoyuki Kasahara
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <stdio.h>

#include "ndtpd.h"
#include <eb/eb.h>
#include <eb/appendix.h>
#include <eb/text.h>

/*
 * Unexported functions.
 */
#ifdef __STDC__
static int hook_narrow_font(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
static int hook_wide_font(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
static int hook_newline(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
static int hook_reference(EB_Book *, EB_Appendix *, char *, EB_Hook_Code,
    int, const int *);
#else /* not __STDC__ */
static int hook_narrow_font();
static int hook_wide_font();
static int hook_newline();
static int hook_reference();
#endif /* not __STDC__ */


/*
 * Text hooks.
 */
static const EB_Hook text_hooks[] = {
    {EB_HOOK_NARROW, eb_hook_euc_to_ascii},
    {EB_HOOK_NARROW_FONT, hook_narrow_font},
    {EB_HOOK_WIDE_FONT, hook_wide_font},
    {EB_HOOK_NEWLINE, hook_newline},
    {EB_HOOK_STOPCODE, eb_hook_stopcode_mixed},
    {EB_HOOK_END_REFERENCE, hook_reference},
    {EB_HOOK_END_MENU, hook_reference},
    {EB_HOOK_NULL, NULL},
};


/*
 * Initialize text hook set.
 */
void
initialize_text_hookset()
{
    eb_initialize_hookset(&text_hookset);
    eb_set_hooks(&text_hookset, (EB_Hook *)text_hooks);
}


/*
 * Hook for narrow font characters.
 */
static int
hook_narrow_font(book, appendix, workbuf, code, argc, argv)
    EB_Book *book;
    EB_Appendix *appendix;
    char *workbuf;
    EB_Hook_Code code;
    int argc;
    const int *argv;
{
    if (eb_narrow_character_text(&current_book->appendix, argv[0], workbuf)
	< 0)
	sprintf(workbuf, "#%04x", argv[0]);

    return 0;
}


/*
 * Hook for wide font characters.
 */
static int
hook_wide_font(book, appendix, workbuf, code, argc, argv)
    EB_Book *book;
    EB_Appendix *appendix;
    char *workbuf;
    EB_Hook_Code code;
    int argc;
    const int *argv;
{
    if (eb_wide_character_text(&current_book->appendix, argv[0], workbuf)
	< 0)
	sprintf(workbuf, "#%04x", argv[0]);

    return 0;
}


/*
 * Hook for a newline character.
 */
static int
hook_newline(book, appendix, workbuf, code, argc, argv)
    EB_Book *book;
    EB_Appendix *appendix;
    char *workbuf;
    EB_Hook_Code code;
    int argc;
    const int *argv;
{
    char *p = workbuf;

    *p++ = '\n';
    *p = '\0';

    return 0;
}


/*
 * Hook for a reference to text.
 */
static int
hook_reference(book, appendix, workbuf, code, argc, argv)
    EB_Book *book;
    EB_Appendix *appendix;
    char *workbuf;
    EB_Hook_Code code;
    int argc;
    const int *argv;
{
    sprintf(workbuf, "<%x:%x>", argv[1], argv[2]);
    return 0;
}
